use serde::Deserialize;
use std::collections::HashMap;
use std::fs;
use std::path::PathBuf;

#[derive(Deserialize, Debug, Clone)]
#[serde(default)]
struct Core {}

impl Default for Core {
    fn default() -> Self {
        Core {}
    }
}

#[derive(Deserialize, Debug, Clone)]
#[serde(default)]
pub struct Remote {
    pub url: Option<String>,
    /// Force connection to nextcloud to be http (not https)
    forceinsecure: bool,
}

impl Default for Remote {
    fn default() -> Self {
        Remote {
            url: None,
            forceinsecure: false,
        }
    }
}

#[derive(Deserialize, Debug, Clone)]
#[serde(default)]
pub struct NsConfig {
    core: Core,
    remote: std::collections::HashMap<String, Remote>,
}

impl NsConfig {
    pub fn get_remote(&self, name: &str) -> Remote {
        self.remote.get(name).unwrap().clone()
    }
}

impl Default for NsConfig {
    fn default() -> Self {
        NsConfig {
            core: Core::default(),
            remote: HashMap::new(),
        }
    }
}

pub fn load_nsconfig(repo_root: &PathBuf) -> NsConfig {
    let mut config_file_path = repo_root.clone();
    config_file_path.push(".nextsync");
    config_file_path.push("config");

    if !config_file_path.exists() {
        return NsConfig::default();
    }

    let config: NsConfig = toml::from_str(&fs::read_to_string(config_file_path).unwrap()).unwrap();

    config
}
