use crate::config::config::Config;
use reqwest::{Client, ClientBuilder};
use reqwest::{header::HeaderMap, Method, Url};

const USER_AGENT: &str = "Nextsync";

pub struct ClientConfig {
    client: Option<ClientBuilder>,
}

impl ClientConfig {
    pub fn new() -> Self {
        ClientConfig {
            client: Some(Client::builder().user_agent(USER_AGENT)),
        }
    }

    pub fn default_headers(mut self, headers: HeaderMap) -> Self {
        self.client = Some(
            self.client
                .take()
                .expect("Client was already built")
                .default_headers(headers),
        );
        self
    }

    pub fn build(&mut self) -> Client {
        self.client
            .take()
            .expect("Cannot build the client twice")
            .build()
            .unwrap()
    }
}

pub struct Service {
    client: ClientConfig,
    config: Option<Config>,
    method: Option<Method>,
    url: Option<String>,
    headers: HeaderMap,
    body: Option<String>,
}

impl Service {
    pub fn new() -> Self {
        Service {
            client: ClientConfig::new(),
            config: None,
            method: None,
            url: None,
            headers: HeaderMap::new(),
            body: None,
        }
    }

    pub fn set_config(&mut self, config: Config) {
        self.config = Some(config);
    }

    pub fn propfind(&mut self, url: String) -> &mut Self {
        self.method = Some(Method::from_bytes(b"PROPFIND").expect("Cannot be invalid"));
        self.url = Some(url);
        self
    }

    pub async fn send(&mut self) -> Result<reqwest::Response, reqwest::Error> {
        let mut url = self
            .config
            .clone()
            .expect("A config must be provided to service")
            .get_nsconfig()
            .get_remote("origin")
            .url
            .expect("An url must be set on the remote");
        url.push_str(&self.url.clone().unwrap());

        self.client
            .build()
            .request(self.method.clone().expect("Method must be set"), url)
            .bearer_auth("rK5ud2NmrR8p586Th7v272HRgUcZcEKIEluOGjzQQRj7gWMMAISFTiJcFnnmnNiu2VVlENks")
            .send()
            .await
    }
}
